/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package br.com.novatrix.tomada.interruptor;

import br.com.globalcode.eletronlivre.arduino.serial.Arduino;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.GetMethod;

/**
 * Thread que busca a cada 1 segundo o valor do Webservice e caso tenha alterado
 * manda informação para o arduino
 * @author jeffprestes
 */
public class Chaveador extends Thread {

    private Resposta resposta;
    private String email;
    private String senha;
    private String portaArduino;
    private HttpClient cliente;
    private HttpMethod metodo;

    public Chaveador(String email, String senha, String portaArduino)   {
        if (email.trim().length()<3)    {
            throw new IllegalArgumentException("Você deve informar um email");
        }

        if (senha.trim().length()<2)    {
            throw new IllegalArgumentException("Você deve informar uma senha.");
        }

        if (portaArduino.trim().length()<2)     {
            throw new IllegalArgumentException("Você deve informar a porta onde esta conectado o Arduino");
        }

        this.setEmail(email);
        this.setSenha(senha);
        this.setPortaArduino(portaArduino);
        
        HttpClient client = new HttpClient();
        HttpMethod metodo = new GetMethod("http://www.novatrix.com.br/tomadaphp/acoes.php?acao=acessar&debug=0&origem=webservice&destino=acoes.php&descEmail=" + this.getEmail() + "&descSenha=" + this.getSenha());
        this.setCliente(client);
        this.setMetodo(metodo);

        Arduino.init(portaArduino);
        Arduino.delay(500);

    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getPortaArduino() {
        return portaArduino;
    }

    public void setPortaArduino(String portaArduino) {
        this.portaArduino = portaArduino;
    }

    public String getSenha() {
        return senha;
    }

    public void setSenha(String senha) {
        this.senha = senha;
    }

    public HttpClient getCliente() {
        return cliente;
    }

    public void setCliente(HttpClient cliente) {
        this.cliente = cliente;
    }

    public HttpMethod getMetodo() {
        return metodo;
    }

    public void setMetodo(HttpMethod metodo) {
        this.metodo = metodo;
    }



    public void run()    {

        HttpClient client = this.getCliente();
        HttpMethod method = this.getMetodo();

        try {
          while(true)   {
              // Execute the method.
              int statusCode = client.executeMethod(method);

              if (statusCode != HttpStatus.SC_OK) {
                System.err.println("Metodo falhou: " + method.getStatusLine());
              }

              // Read the response body.
              byte[] responseBody = method.getResponseBody();

              String retornoWS = new String(responseBody);

              // Deal with the response.
              // Use caution: ensure correct character encoding and is not binary data
              Resposta resp = Resposta.getResposta(retornoWS);

              if (resp.getStatus() == Resposta.Status.OK)   {
                  
                  //System.out.println(retornoWS + ":" + resp.getStatus() + ":" + resp.getPosicaoChave() );

                  if (resposta==null)   {
                        System.out.println("Estado atual do interruptor " + resp.getPosicaoChave().getDescricao());
                        resposta = new Resposta(Resposta.Status.ERRO, Resposta.PosicaoChave.DESLIGADO);
                  }

                  if (resp.getPosicaoChave() != resposta.getPosicaoChave())     {
                      resposta = resp;
                      System.out.println("Aciona rele para " + resposta.getPosicaoChave().getDescricao());

                      try   {
                          if (resposta.getPosicaoChave().getCodigo()==1)    {
                                Arduino.enviar("A".getBytes());
                          }     else if (resposta.getPosicaoChave().getCodigo()==0)    {
                                Arduino.enviar("B".getBytes());
                          }
                      } catch (Exception ex)        {
                          ex.printStackTrace();
                      }

                  }
              }
              
              // Release the connection.
              method.releaseConnection();
              Thread.sleep(1000);
          }

        } catch (InterruptedException ex) {
          System.err.println("Thread interrompida: " + ex.getMessage());
          ex.printStackTrace();
        } catch (HttpException e) {
          System.err.println("Violacao do protocolo: " + e.getMessage());
          e.printStackTrace();
        } catch (IOException e) {
          System.err.println("Erro de transporte: " + e.getMessage());
          e.printStackTrace();
        } finally {
          // Release the connection.
          method.releaseConnection();
        }
    }

    

}
